export function scrollToFragment(options) {
    if (options === void 0) { options = {}; }
    var _a, _b;
    unmount();
    currentOptions = {
        getElement: (_a = options.getElement) !== null && _a !== void 0 ? _a : getElementById,
        history: options.history,
        scrollIntoView: (_b = options.scrollIntoView) !== null && _b !== void 0 ? _b : scrollIntoView,
    };
    mount();
}
function mount() {
    var _a;
    documentObserver = new MutationObserver(handleDomMutation);
    document.addEventListener("click", handleDocumentClick);
    unlistenHistory = (_a = currentOptions.history) === null || _a === void 0 ? void 0 : _a.listen(handleHistoryPush);
    startObserving();
}
function unmount() {
    stopObserving();
    document.removeEventListener("click", handleDocumentClick);
    if (unlistenHistory)
        unlistenHistory();
    unlistenHistory = undefined;
    documentObserver = undefined;
}
function startObserving() {
    stopObserving();
    if (!location.hash)
        return;
    addEventListener("touchend", stopObserving);
    addEventListener("wheel", stopObserving);
    document.addEventListener("selectstart", stopObserving);
    documentObserver === null || documentObserver === void 0 ? void 0 : documentObserver.observe(document, OBSERVER_CONFIG);
    adjustScrollPosition();
    observeTimeout = setTimeout(stopObserving, OBSERVE_TIMEOUT_MS);
}
function stopObserving() {
    clearTimeout(observeTimeout);
    cancelAnimationFrame(throttleRequestId);
    documentObserver === null || documentObserver === void 0 ? void 0 : documentObserver.disconnect();
    removeEventListener("touchend", stopObserving);
    removeEventListener("wheel", stopObserving);
    document.removeEventListener("selectionchange", stopObserving);
}
function handleHistoryPush(_location, action) {
    if (action === "PUSH")
        startObserving();
    else
        stopObserving();
}
function handleDocumentClick(event) {
    if (event.target.nodeName !== "A")
        return;
    var anchor = event.target;
    if (anchor.href.indexOf("#") === -1)
        return;
    if (anchor.href.replace(/#.*/, "") === location.href.replace(/#.*/, "")) {
        throttle(startObserving);
    }
}
function handleDomMutation() {
    throttle(adjustScrollPosition);
}
function adjustScrollPosition() {
    if (!location.hash)
        return;
    var fragmentId = decodeURIComponent(location.hash.substring(1));
    var element = currentOptions.getElement.call(null, fragmentId);
    if (element)
        currentOptions.scrollIntoView.call(null, element);
}
function getElementById(id) {
    return document.getElementById(id);
}
function scrollIntoView(element) {
    element.scrollIntoView();
}
function throttle(callback) {
    cancelAnimationFrame(throttleRequestId);
    throttleRequestId = requestAnimationFrame(callback);
}
var currentOptions;
var unlistenHistory;
var documentObserver;
var observeTimeout;
var throttleRequestId;
var OBSERVER_CONFIG = {
    attributes: true,
    characterData: true,
    childList: true,
    subtree: true,
};
var OBSERVE_TIMEOUT_MS = 10000;
//# sourceMappingURL=index.js.map